body, html {
    padding: 0;
    margin: 0;
}

body {
    background-color: #f0f0f0;
    font-family: 'Open Sans', sans-serif;
}

div.hero {
    background-color: #4C89C8;
    box-sizing: border-box;
    padding: 5rem;
}

div.hero h1 {
    color: white;
    font-weight: normal;
    text-align: center;
}

section {
    padding-bottom: 2rem;
}

div.content {
    background-color: white;
    border-radius: 0.5rem;
    box-sizing: border-box;
    margin: 1rem auto;
    max-width: 50rem;
    padding: 2rem;
    transform: translateY(-4rem);
}

div.content h2 {
    font-weight: normal;
    border-bottom: 4px solid #f0f0f0;
}

p.code {
    background-color: #2b2b2b;
    color: white;
    font-family: monospace;
    padding: 1rem;
    margin: 0;
    display: block;
}

p.code span {
    display: block;
    padding: 0.5rem;
}